/*!
 * @license New BSD License
 * @copyright Copyright (c) 2010, Nokia Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the Nokia Corporation nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NOKIA CORPORATION BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef MAPWIDGET_H
#define MAPWIDGET_H

#include <QGraphicsWidget>
#include "GeoEngine.h"

class MapWidgetPrivate;

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

/*!
    \class MapWidget
    \brief The MapWidget class provides map widget object supports basic interactions,
    geo coordinate convertion to pixel coordinate and functionality to add objects on top of map widget.

    MapWidget object provides with map widget object supports basic interactions like panning, zooming, pinch gesture.
    It also provides with geo coordinate convertion to pixel coordinate and functionality to add objects ontop of map widget.
    MapWidget requires GeoEngine object to be created and initialized.
    MapWidget requires main loop to be started.
*/

class MapWidget : public QGraphicsWidget
{
    Q_OBJECT
    Q_PROPERTY(MapWidget::PanningMode panningMode READ panningMode WRITE setPanningMode)
    Q_PROPERTY(MapWidget::PinchMode pinchMode READ pinchMode WRITE setPinchMode)
    Q_PROPERTY(QGeoCoordinate center READ center WRITE moveTo)
    Q_PROPERTY(int zoomLevel READ zoomLevel WRITE setZoomLevel)
    Q_PROPERTY(int scaleLevel READ scaleLevel)
    Q_PROPERTY(bool perspective READ perspective WRITE setPerspective)
    Q_ENUMS(PanningMode)
    Q_ENUMS(PinchMode)

public:
    enum PanningMode {
        NoPanning,
        PoorPanning,
        LeastPanning,
        FullPanning
    };

    enum PinchMode {
        PoorPinch,
        LeastPinch
    };

    /*!
    * Constructor.
    * @param geoEngine GeoEngine
    * @param parent
    * @param aPaintingMode
    */
    MapWidget(GeoEngine *geoEngine, QGraphicsWidget *parent = 0);

    /*!
    * Destructor.
    */
    ~MapWidget();

    /*!
    * Returns a pointer to geo engine.
    */
    GeoEngine* engine() const;

    /*!
    * Returns panning mode.
    */
    MapWidget::PanningMode panningMode() const;

    /*!
     * Returns pinch mode.
     */
    MapWidget::PinchMode pinchMode() const;

    /*!
     * Returns current map center.
     */
    QGeoCoordinate center() const;

    /*!
     * Returns current zoom level.
     * @return current zoom level or -1 in case of an error.
     */
    int zoomLevel() const;

    /*!
     * Returns current scale level (centimeters per pixel).
     * @return current scale level or -1 in case of an error.
     */
    int scaleLevel() const;

    /*!
     * Returns minimal zoom level.
     * @return minimal zoom level or -1 in case of an error.
     */
    int minZoomLevel() const;

    /*!
     * Returns maximal zoom level.
     * @return maximal zoom level or -1 in case of an error.
     */
    int maxZoomLevel() const;

    /*!
     * Converts zoom level value to corresponding zoom scale value.
     * @return the zoom scale value for given zoom level. In case of errors it returns -1.
     * @param zoomLevel value of zoom level to convert
     */
    int zoomLevelToScale(int zoomLevel) const;

    /*!
     * Returns true if 3D mode enabled, returns false otherwise.
     */
    bool perspective() const;

    /*!
     * Add geotagged widget. This MapWidget takes ownership of the widget.
     * @param widget to be mapped
     * @param coord place to be tagged
     * @param anchor point in widget's coordinate system to be mapped
     * @return true if added.
     */
    bool addGeoWidget(QGraphicsObject *widget, const QGeoCoordinate &coord, const QPointF &anchor = QPointF());

    /*!
     * Update widget's coordinate and anchor.
     * @param widget to be updated
     * @param coord place to be tagged
     * @param anchor point in widget's coordinate system to be mapped
     * @return true if this MapWidget owns widget and new properties are set succesfully.
     */
    bool updateGeoWidget(QGraphicsObject *widget, const QGeoCoordinate &coord, const
                         QPointF &anchor = QPointF());

    /*!
     * Remove geotagged widget. Invalidates widget's parent and corresponding properties.
     * @return true if this MapWidget owns widget and widget is removed.
     */
    bool removeGeoWidget(QGraphicsObject *widget);

    /*!
     * Converts GeoCoordinate to pixel coordinate
     * @return QPoint.
     * \attention At startup and after resize wait for the ready() signal prior to use this API.
     */
    QPoint toPixel(const QGeoCoordinate &coord);

    /*!
     * Converts pixel coordinate to GeoCoordinate
     * @return GeoCoordinate.
     * \attention At startup and after resize wait for the ready() signal prior to use this API.
     */
    QGeoCoordinate toGeo(const QPoint &point);

public slots:
    /*!
     * Zoom in by step.
     */
    void zoomIn();

    /*!
     * Zoom out by step.
     */
    void zoomOut();

    /*!
     * Enables 3D mode.
     * @param enabled set true to enable 3D mode and false to disable.
     */
    void setPerspective(bool enabled);

    /*!
     * Sets panning mode.
     * @param mode Panning mode.
     * \attention MapWidget::PoorPanning is no longer supported and now idential to MapWidget::LeastPanning.
     */
    void setPanningMode(MapWidget::PanningMode mode);

    /*!
     * Sets pinch mode.
     * @param mode Pinch mode.
     * \attention MapWidget::PoorPinch is no longer supported and now idential to MapWidget::LeastPinch.
     */
    void setPinchMode(MapWidget::PinchMode mode);

    /*!
     * Center the map to the specified coordinate.
     * @param coord The coordinate the map center to be moved to.
     * \attention Wait for moveFinished() signal to be sure the map has been moved.
     */
    void moveTo(const QGeoCoordinate &coord);

    /*!
     * Set zoom to predefined discrete level.
     */
    void setZoomLevel(int zoomLevel);

signals:
    /*!
     * Signal to notify the zoom level has been changed.
     */
    void zoomLevelChanged();

    /*!
     * Signal to notify the scale level has been changed.
     */
    void scaleLevelChanged();

    /*!
     * Signal to notify the map has been moved.
     */
    void moveFinished();

    /*!
     * Signal to notify the widget is ready and it is safe 
     * to call toPixel() and toGeo() APIs.
     */
    void ready();

protected:
    void resizeEvent (QGraphicsSceneResizeEvent *event);
    bool event(QEvent *event);

private:
    Q_DISABLE_COPY(MapWidget)
    Q_DECLARE_PRIVATE(MapWidget)
    MapWidgetPrivate *d_ptr;
};

QT_END_NAMESPACE

QT_END_HEADER


#endif  //MAPWIDGET_H

